# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script uses an ML pipeline to train and serve an XGBoost Model.
"""

import xgboost as xgb

from typing import Tuple

from sklearn.model_selection import train_test_split
from sklearn.datasets import fetch_california_housing

from kale.types import MarshalData
from kale.sdk import pipeline, step
from kale.common import mlmdutils, artifacts


def _postprocess(inputs):
    """Postprocess the prediction value."""
    value = inputs["outputs"][0]["data"][0]
    inputs["outputs"][0]["data"][0] = round(value * 100000, 2)
    return inputs


@step(name="data_loading")
def load_split_dataset() -> Tuple[MarshalData, MarshalData]:
    """Fetch California Housing dataset."""
    # get data and target of the dataset
    california_housing = fetch_california_housing()
    x = california_housing.data
    y = california_housing.target

    # split the dataset
    x_train, _, y_train, _ = train_test_split(x, y, test_size=.2,
                                              random_state=42)
    return x_train, y_train


@step(name="register_transformer")
def register_transformer() -> int:
    """Register a Transformer artifact."""
    # create and submit a Transformer artifact
    mlmd = mlmdutils.get_mlmd_instance()

    transformer_artifact = artifacts.Transformer(
        name="Transformer",
        postprocess_fn=_postprocess
    ).submit_artifact()

    mlmd.link_artifact_as_output(transformer_artifact.id)

    return transformer_artifact.id


@step(name="model_training")
def train(x: MarshalData, y: MarshalData) -> MarshalData:
    """Train a XGBRegressor model."""
    model = xgb.XGBRegressor(objective='reg:squarederror',
                             colsample_bytree=1,
                             eta=0.3,
                             learning_rate=0.1,
                             max_depth=5,
                             alpha=10,
                             n_estimators=2000)
    model.fit(x, y)
    return model


@pipeline(name="regression", experiment="xgboost-tutorial")
def ml_pipeline():
    """Run the ML pipeline."""
    x_train, y_train = load_split_dataset()
    register_transformer()
    train(x_train, y_train)


if __name__ == "__main__":
    ml_pipeline()
